#    Diagnostic and test task for the AGATA Xport interface

set frame  Xport
set title "Xport Client"
set icon   Xport

set started 0

#package require XportClient


set LogFlag 1

set inputcolour  green4
set outputcolour red
set insertcolour blue
set bkgcolour    azure

set inputprompt  ""
set outputprompt ""
set insertprompt ">>"

set histlim 40
for {set i 0} {$i <= $histlim} {incr i} {lappend history ""}
set nextcommand $histlim

set XportServerList {localhost 193.62.115.238}
set XportServer [lindex $XportServerList 0]

set XportPortList {10001}
set XportPort [lindex $XportPortList 0]

set requestchoice     {"Simple Write" "Long Write" "Read"}
set destinationchoice {Core Segment}
set CoreDestinationchoice   {"Virtex for Segment ADC card 1" "Virtex for Segment ADC card 2" "Virtex for Core ADCs" "Main Board"}
set SegmentDestinationchoice   {"Virtex for Segment ADC card 1" "Virtex for Segment ADC card 2" "Virtex for Segment ADC card 3" \
                   "Segment ADC card 4" "Main Board"}

set request 0
set destination 0
set command0 0
set command1 0
set data0 0
set data1 0

set verbose 0


proc make-frame {} {
   global env XportServerList XportPortList
   global requestchoice destinationchoice CoreDestinationchoice

set rows 24
set cols 80

  :frame [self frame-name] :label "[self frame-title]" \
     :icon-name "[self icon-title]"

  :panel P0 :layout V :padx 5 :pady 5
    :layout H; :col-gap 0
    :text XPORTADDRT         :label "Xport Device/IP address" :w 20
    :choice-stack XPORTADDRC :label "" :strings $XportServerList
    :hskip 50
    :text XPORTPORTT         :label "TCP Port" :w 12 
    :choice-stack XPORTPORTC :label "" :strings $XportPortList

  :panel P1 :layout V :padx 5 :pady 5
    :layout H; :col-gap 20
     :hskip 400
     :button CLEAR   :label "Clear log window" :command ":modify LOG :empty T"
     :checkbox VERBOSE :label "Enable Verbose Option" 

  :panel P2 :layout V :padx 5 :pady 5
    :galley LOG :rows $rows :cols $cols :font variable
     configuretags

  :panel P3 :layout V :padx 5 :pady 5
      :layout H; :row-gap 10; :col-gap 30
      :choice-stack REQUEST     :label "Request Type" :exclusive T :strings $requestchoice :value 0
      :choice-stack DESTINATION :label "Destination"  :exclusive T :strings $destinationchoice :value 0
      :choice-stack COMMAND0    :label "Command Byte 0"  :exclusive T :strings $CoreDestinationchoice :value 0

       :next-row T

      :text COMMAND1  :label "Command Byte 1" :w 8 :value 0
      :text DATA0     :label "Data Byte 0"    :w 8 :value 0
      :text DATA1     :label "Data Byte 1"    :w 8 :value 0
      :button EXECUTE :label "Execute Command" :w 24

   :panel P4 :layout T :padx 10 :pady 10
     :layout H; :col-gap 50; :row-gap 10
#     :button BAT :label "Execute file"
#     :menubutton HISTORY   :label "History"  :strings [list ""]
#     :button VERBOSE :label "Set Server Verbose" :command set_verbose
#     :next-row T
#     :colour local entry foreground black
#     :text CMD  :label "command:" :w [expr $cols - 10]
#    bind [:path CMD].entry <Meta-k>         "restorecommand last"
#    bind [:path CMD].entry <Meta-BackSpace> "zapcommand"
#    bind [:path CMD].entry <Alt-KeyPress-k>         "restorecommand last"
#    bind [:path CMD].entry <Alt-KeyPress-BackSpace> "zapcommand"
#    bind [:path CMD].entry <Up> "restorecommand previous"
#    bind [:path CMD].entry <Down> "restorecommand next"

   open-frame

  :galley_size LOG $rows $cols

#    focus [:path CMD].entry

   :show T
}


proc click-configure {W} {:galley_resize LOG}

proc restorecommand {c} {
  global history histlim nextcommand 
  switch $c {
    last  {:modify CMD :value [lindex $history end]}
    previous  {
     :modify CMD :value [lindex $history $nextcommand]
     incr nextcommand -1
     if {$nextcommand < 0} then {set nextcommand 0}
    }
    next  {
     incr nextcommand 1
     if {$nextcommand > $histlim} then {set nextcommand $histlim}
     :modify CMD :value [lindex $history $nextcommand]
    }
    default {insert "restorecommand $c?"}
  }
}

proc zapcommand {} {:modify CMD :value ""}

proc configuretags {} {
   global inputcolour outputcolour insertcolour bkgcolour

    :modify LOG :createtag "taginput  -foreground $inputcolour  -background $bkgcolour"
    :modify LOG :createtag "tagoutput -foreground $outputcolour -background $bkgcolour"
    :modify LOG :createtag "taginsert -foreground $insertcolour -background $bkgcolour"
}

proc click-bat args {
    global env XportServer DiagFileBase

#    Here use the built in browser function
#    This uses the canvas function and requires a pale coloured background

    option add *Canvas.background  #c0c020
    set types {
      {{Command Files}      {.bat}   }
      {{All Files}   *        }
    }
    set batfile [tk_getOpenFile -parent [self frame-name] -title "Diagnostic script Selection"  \
       -initialdir $DiagFileBase -filetypes $types]
    option add *Canvas.background  $env(tkcanvasbackground)
    if {$batfile != ""} then {
       set directory [file dirname $batfile]
       set file [file tail $batfile]
       set z [midas-confirmation "Execute file $file for the VME card in VME crate $XportServer"]
       if {$z == 0} then {return}
       uplevel #0 source $batfile
       set DiagFileBase $directory
    }
}


proc invoke {text} {
    global outputprompt inputprompt
    global history histmenu histlim nextcommand
    global AbortFlag LogFlag

    set AbortFlag 0

    set cmd ""
    foreach item [split $text ", "] {
        if {$item != ""} then {append cmd $item " "}
    }

    :modify LOG :usingtag taginput :strings "[set inputprompt]$cmd\n"

  if [info complete $cmd] {
       if {$cmd == "!!"} {
           set cmd [lindex $history end]
       } else {
           lappend history "$cmd"
           set history [lreplace $history 0 0]
       }
  
    if {[first $cmd] == "repeat"} then {
        set count [second $cmd]
        set l [string length "repeat $count"]
        set cmd [string range $cmd [expr $l+1] end]
    } else {set count 1}

    if {$count == 0} then {set count 100000}
    for {set i 1} {$i <= $count} {incr i} {
       update
       if {$AbortFlag == 1} {:modify LOG :strings "Abort\n"; break}
       set result [catch {eval $cmd} msg]
       if {$result != 0} {
            :modify LOG :usingtag tagoutput :strings "[set outputprompt]Error: $msg\n"
            if {$i <= $count} then {:modify LOG :usingtag tagoutput :strings "loop aborted due to error\n"}
            break;
       } else {
           if {$msg != ""} then {if {$LogFlag == 1 || $i == 1} then {:modify LOG :usingtag tagoutput :strings "$outputprompt$msg\n"}}
       }
    }
  }

    :modify CMD :value ""
    set nextcommand $histlim

    set histmenu {}
    set i 0
    foreach s $history {
       if {$s != ""} then {lappend histmenu "$i [string range "$s" 0 40]"}
       incr i
    }
    :modify HISTORY :strings $histmenu
}


proc click_load {} {midas-browser R do_load}

proc do_load {args} {
   upvar #0 browserdirectory directory
   upvar #0 browserfile file

   set f [file join $directory $file]
   cd $directory
   insert "loading $f"
   set z [catch {source $f} m]
   if {$z != 0} then {insert "$m"}
}

proc do_command {W s} {
   global CurrentCommand LastMethodName
   set CurrentCommand "$s"
   set LastMethodName [first $CurrentCommand]
   if [info complete $s] {
       set z [catch {eval $s} m]
       if {$z != 0} then {insert "$m"} else {insert "$s=> $m"}
   }
   :modify CMD :value ""
}

proc restorecommand {args} {
    global CurrentCommand
    :modify CMD :value "$CurrentCommand"

}

proc insert {s} {:modify LOG :usingtag taginsert :strings $s\n}


proc start args {
   global started XportServerList XportPortList

   if {$started == 0} then {make-frame} else {open-frame}
   set started 1
   
   click-server [lindex $XportServerList 0]
   click-port [lindex $XportPortList 0]
}

proc click arg {
   global history verbose
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   set i [lindex $arg 4]

   clear-footer
   switch $w  {

      XPORTADDRT    {click-server $v}
      XPORTADDRC    {click-server $i}
      XPORTPORTT    {click-port $v}
      XPORTPORTC    {click-port $i}

      REQUEST       {click-request $v}
      DESTINATION   {click-destination $v}
      COMMAND0      {click-command0 $v}
      COMMAND1      {click-command1 $v}
      DATA0         {click-data0 $v}
      DATA1         {click-data1 $v}

      VERBOSE       {set verbose $v}


      CMD        {invoke "$v"}
      HISTORY    {
                 set i [first $v]
                 invoke "[lindex $history $i]"
                 }
 
      default {
         set p "click-[lowercase $w]"
         if {[info procs $p] == "$p"} then {$p $v} else {set-footer "no handler for widget $w"}
      }
   }
}

proc click-server {v} {
    global XportServerList XportServer

    set  XportServer $v
    :modify XPORTADDRT :value $XportServer
    :modify XPORTADDRC :value [lsearch $XportServerList $XportServer]

}

proc click-port {v} {
    global XportPortList XportPort

    set  XportPort $v
    :modify XPORTPORTT :value $XportPort
    :modify XPORTPORTC :value [lsearch $XportPortList $XportPort]

}

proc click-request {v} {
    global request destination

    set request $v
    set byte0 [format "0x%x" [expr ($destination << 7) + ($request << 5)]]

    insert "destination byte= $byte0"
}

proc click-destination {v} {
    global request destination CoreDestinationchoice SegmentDestinationchoice

    set destination $v
    set byte0 [format "0x%x" [expr ($destination << 7) + ($request << 5)]]

    insert "destination byte= $byte0"

    switch $v {
        0   {:modify COMMAND0 :strings $CoreDestinationchoice}
        1   {:modify COMMAND0 :strings $SegmentDestinationchoice}
    }

    click-command0 0
}

proc click-command0 {v} {
    global request destination command0

    :modify COMMAND0 :value $v

    set command0 $v
    set byte0 [format "0x%x" [expr ($destination << 7) + ($request << 5) + ($command0 << 2)]]

    insert "command 0 byte= $byte0"
}

proc click-command1 {v} {
    global command1

    set command1 $v
    set byte0 [format "0x%x" $command1]

    insert "command 1 byte= $byte0"
}

proc click-data0 {v} {
    global data0

    set data0 $v
    set byte0 [format "0x%x" $data0]

    insert "data 0 byte= $byte0"
}

proc click-data1 {v} {
    global data1

    set data1 $v
    set byte0 [format "0x%x" $data1]

    insert "data 1 byte= $byte0"
}

proc click-execute {args} {
    global request

    switch $request {
        0   {do_write}
        1   {do_longwrite}
        2   {do_read}
    }

}

proc do_write {} {
    global XportServer XportPort
    global request destination command0 command1 data0 data1 verbose

    set byte0 [format "0x%x" [expr ($destination << 7) + ($request << 5)]]
    set byte1 0x0
    set byte2 0x0
    set byte3 0x4
    set byte4 [format "0x%x" [expr ($destination << 7) + ($request << 5) + ($command0 << 2)]]
    set byte5 [format "0x%x" $command1]
    set byte6 [format "0x%x" $data0]
    set byte7 [format "0x%x" $data1]

    insert "sending command to $XportServer port $XportPort : $byte0 $byte1 $byte2 $byte3 $byte4 $byte5 $byte6 $byte7"

    set z [catch {exec xport_cmd -n $XportServer -p $XportPort -v $verbose $byte0 $byte1 $byte2 $byte3 $byte4 $byte5 $byte6 $byte7} m]
    if {$z == 0} then {
        insert "received good response:\n $m"
        set response [last [split $m \n]]
        if {[llength $response] == 4} then {insert "received Successful Write ACK"; return}
        if {[llength $response] == 6} then {insert "received Failed Write ACK at [lindex $response 4] [lindex $response 5]"; return}
        insert "received unexpected reply: $response"
    } else {
        insert "received error response: rc=$z\n  $m"
    }
}

proc do_read {} {
    global XportServer XportPort
    global request destination command0 command1 data0 data1 verbose

    set byte0 [format "0x%x" [expr ($destination << 7) + ($request << 5)]]
    set byte1 0x0
    set byte2 0x0
    set byte3 0x4
    set byte4 [format "0x%x" [expr ($destination << 7) + ($request << 5) + ($command0 << 2)]]
    set byte5 [format "0x%x" $command1]
    set byte6 0x0
    set byte7 0x0

    insert "sending command to $XportServer port $XportPort : $byte0 $byte1 $byte2 $byte3 $byte4 $byte5 $byte6 $byte7"

    set z [catch {exec xport_cmd -n $XportServer -p $XportPort -v $verbose $byte0 $byte1 $byte2 $byte3 $byte4 $byte5 $byte6 $byte7} m]
    if {$z == 0} then {
        insert "received good response:\n $m"
        set response [last [split $m \n]]
        if {[llength $response] > 6} then {insert "received Successful Read ACK\n$response"; return}
        insert "received unexpected reply: $response"
    } else {
        insert "received error response: rc=$z\n  $m"
    }
}

proc do_longwrite {} {
    global XportServer XportPort
    global request destination command0 command1 data0 data1 verbose

    set byte0 [format "0x%x" [expr ($destination << 7) + ($request << 5)]]
    set byte1 0x0
    set byte2 0x0
    set byte3 0x4
    set byte4 [format "0x%x" [expr ($destination << 7) + ($request << 5) + ($command0 << 2)]]
    set byte5 [format "0x%x" $command1]
    set byte6 [format "0x%x" $data0]
    set byte7 [format "0x%x" $data1]

    insert "sending command to $XportServer port $XportPort : $byte0 $byte1 $byte2 $byte3 $byte4 $byte5 $byte6 $byte7"

    set z [catch {exec xport_cmd -n $XportServer -p $XportPort -v $verbose $byte0 $byte1 $byte2 $byte3 $byte4 $byte5 $byte6 $byte7} m]
    if {$z == 0} then {
        insert "received good response:\n $m"
        set response [last [split $m \n]]
        if {[llength $response] == 4} then {insert "received Successful Write ACK"; return}
        if {[llength $response] == 6} then {insert "received Failed Write ACK at [lindex $response 4] [lindex $response 5]"; return}
        insert "received unexpected reply: $response"
    } else {
        insert "received error response: rc=$z\n  $m"
    }

}
